#ifndef BOOLEAN_MATCHER_HPP_INCLUDED
#define BOOLEAN_MATCHER_HPP_INCLUDED

#include <map>
#include <set>

#include "parser.hpp"
#include "acmachine.hpp"
#include "expressions.hpp"
#include "result_params.h"
#include "matchstruct.hpp"

class BooleanMatcher 
{
public:
    BooleanMatcher( ExpressionParser& parser, bool wholeWord = false );
    ~BooleanMatcher();

    bool feedCharacter( wchar_t character );
    void fill_pos_list(const stream_range& element_);
    const stream_list& get_pos_list() const;
    void reset();
    void clear();

    bool evaluate() const 
    { 
        return parser_.get()->evaluate(pos_list_);
    };

    MatchType match() const 
    {
        return parser_.get()->match(pos_list_);
    }

    void match(MatchType & result) const 
    { 
        parser_.get()->match(pos_list_, result);
    }

    bool isWholeWord() const 
    { 
        return wholeWord_;
    }

    void resetMatches() 
    {
        parser_.get()->reset();
    }

    void explain(std::wostream& os_) 
    {
        parser_.get()->explain(os_);
    }

    void explain_with_tags(std::wostream& os_) 
    {
        parser_.get()->explain_with_tags(os_);
    }

    unsigned long long get_pos() const 
    {
        return global_pos_;
    }

    bool get_not() const 
    {
        return has_not_;
    }

private:
    BooleanMatcher( const BooleanMatcher& );
    BooleanMatcher& operator= ( const BooleanMatcher& );

private:
    void updateMatches();
    void addLookBehindCharacter( wchar_t character );
    bool lookBehindForSpace( std::size_t position );
    void checkLastMatch( wchar_t character );
    int initializeMachine();

private:
    typedef std::queue<Expression*> LastMatchList;

    bool wholeWord_;
    std::vector<wchar_t> lookbehind_buffer_;
    std::size_t current_lookbehind_position_;
    LastMatchList last_match_;
    ACPMMachine machine_;
    ExpressionParser& parser_;
    unsigned long long local_pos_;
    unsigned long long global_pos_;
    stream_list pos_list_;
    bool has_not_;
};

#endif // BOOLEAN_MATCHER_HPP_INCLUDED
