//
// parser.hpp
//

#ifndef PARSER_HPP_INCLUDED
#define PARSER_HPP_INCLUDED

#include <string>
#include <queue>
#include <map>
#include <set>

#include <string>

#include "parser_impl.hpp"

class KeyWords {
public:
   KeyWords();
   int getToken( const std::wstring& word ) const;

private:
   KeyWords( const KeyWords& );
   KeyWords& operator= ( const KeyWords& );

private:
   typedef std::map< std::wstring, int > KeyWordMap;
   KeyWordMap map_;
}; 

class ExpressionParser : public ExpressionParserImpl {
   typedef ExpressionParserImpl inherited;
public:
   typedef std::set< std::wstring > SoughtsList;
public:
   ExpressionParser( const wchar_t* expression, std::size_t expressionSize, bool caseSensitive = false );
   Expression* get() const { return expression_; }
   bool isCaseSensitive() const { return caseSensitive_; }
   void getSoughtsList( SoughtsList& ) const;
   virtual int yylex( yy_ExpressionParserImpl_stype* );
   virtual void yyerror( char* );
   virtual ~ExpressionParser();

private:
   ExpressionParser( const ExpressionParser& );
   ExpressionParser& operator= ( const ExpressionParser& );

private:
   static const KeyWords keywords_;
   static const wchar_t OPEN = L'(';
   static const wchar_t CLOSE = L')';
   static const wchar_t QUOTE = L'"';
   static const wchar_t SLASH = L'\\';
   static const wchar_t BACK_SLASH = '/';

private:
   struct State { enum Type { Ready, Error, Token, Near, Back_Slash, Distance }; };
   typedef std::queue< Expression* > ExpressionsQueue;
   typedef std::wstring::const_iterator CharIt;

private:
   State::Type readQuoted();
   State::Type readEscaped();
   State::Type readCharacter();
   int parseWord( yy_ExpressionParserImpl_stype* element );

private:
   std::wstring expression_str_;
   CharIt charIt_;
   wchar_t character_;
   std::wstring word_;
   State::Type state_;
   bool quoted_;
};

#endif // PARSER_HPP_INCLUDED
